import copy
import io
import os.path
import sys

import jinja2 as j2
import yaml


def load_output_config(config):
    loaded_output_config = {}

    name = config["name"]
    dest_dir = config["destination_dir"]
    templates = config["templates"]

    if not (os.path.exists(dest_dir) and os.path.isdir(dest_dir)):
        sys.exit(f"'{name}:{dest_dir}' output Destination Dir is not exists!")

    loaded_output_config = copy.deepcopy(config)

    for t_name, t_path in templates.items():
        # @TODO: Use try-except
        if not os.path.exists(t_path):
            sys.exit(f"'{name}:{t_name}:{t_path}' output Template File is not exists!")

        with io.open(t_path) as f:
            data = f.read()

        loaded_output_config["templates"][t_name] = data

    return loaded_output_config


def generate_files(output_config, k8s_services):
    dest_dir = output_config["destination_dir"]
    default_template = output_config["default_template"]
    templates = output_config["templates"]
    extras = output_config["extras"]
    filename_format = output_config["filename_format"]

    for s_name, s_conf in k8s_services.items():
        namespace = s_conf["namespace"]
        node_ips = s_conf["node_ips"]
        node_port = s_conf["node_port"]
        template_type = s_conf["template"]
        if template_type == "":
            template_type = default_template

        filename = filename_format.format(name=s_name, namespace=namespace, **extras)
        filepath = os.path.join(dest_dir, filename)

        template = j2.Template(templates[template_type])

        with io.open(filepath, "w") as f:
            f.write(
                template.render(
                    name=s_name,
                    node_ips=node_ips,
                    node_port=node_port,
                    namespace=namespace,
                    **extras,
                )
            )



import sys

import kubernetes as k8s

PROJECT_DOMAIN = "hayde.f9n.io"

K8S_ANNOTATION_SCHEMA = f"""
{PROJECT_DOMAIN}/enabled: true
{PROJECT_DOMAIN}name: ""
{PROJECT_DOMAIN}/template: http
"""


def get_node_ips_of_k8s_cluster(client, labels={}):
    node_ips = []
    node_objects = client.list_node().items
    for node_object in node_objects:
        _node_labels = node_object.metadata.labels

        shared_items = get_shared_items_between_2_dicts(labels, _node_labels)
        if not len(shared_items.keys()) == len(labels.keys()):
            continue

        for address_object in node_object.status.addresses:
            if address_object.type == "InternalIP":
                node_ips.append(address_object.address)
                continue

    return node_ips


def get_shared_items_between_2_dicts(x, y):
    return {k: x[k] for k in x if k in y and x[k] == y[k]}


# @TODO: Rename this function
def get_k8s_clusters(context_names, node_labels, service_config):
    k8s_clusters = {}

    for context_name in context_names:
        client = k8s.client.CoreV1Api(
            api_client=k8s.config.new_client_from_config(context=context_name)
        )
        cluster_node_ips = get_node_ips_of_k8s_cluster(
            client=client, labels=node_labels
        )
        cluster_services = get_proper_k8s_services(
            client, service_config=service_config
        )
        k8s_clusters[context_name] = {
            "node_ips": cluster_node_ips,
            "services": cluster_services,
        }

    return k8s_clusters


# Ihhhhhh!
def get_proper_k8s_services(client, service_config):
    service_filter_annotations = service_config.get("annotations", {})
    if service_filter_annotations:
        service_filter_annotations = {
            f"{PROJECT_DOMAIN}/{k}": v for k, v in service_filter_annotations.items()
        }

    services = []
    service_objects = client.list_service_for_all_namespaces().items

    for service_object in service_objects:
        s = {}
        s["name"] = service_object.metadata.name
        s["namespace"] = service_object.metadata.namespace

        if not service_object.spec.type == "NodePort":
            continue

        for port_object in service_object.spec.ports:
            s["node_port"] = port_object.node_port

        if not "node_port" in s:
            continue

        if service_filter_annotations == {}:
            services.append(s)
            continue

        _s_annotations = service_object.metadata.annotations

        if not _s_annotations:
            continue

        shared_items = get_shared_items_between_2_dicts(
            service_filter_annotations, _s_annotations
        )

        if not len(shared_items.keys()) == len(service_filter_annotations.keys()):
            continue

        _s_name_on_annotation = _s_annotations.get(f"{PROJECT_DOMAIN}/name", "")
        if not _s_name_on_annotation == "":
            s["name"] = _s_name_on_annotation

        s["template"] = _s_annotations.get(f"{PROJECT_DOMAIN}/template", "")

        services.append(s)

    return services


def get_all_proper_k8s_services(k8s_configs):
    contexts = k8s_configs["contexts"]
    node_labels = k8s_configs["node"]["labels"]
    service_config = k8s_configs.get("service", {})

    k8s_clusters = get_k8s_clusters(
        context_names=contexts, node_labels=node_labels, service_config=service_config
    )

    all_k8s_services = {}
    for k8s_cluster_name, k8s_cluster in k8s_clusters.items():
        for service in k8s_cluster["services"]:
            _service_name = service["name"]

            _s = all_k8s_services.get(
                _service_name, {"node_port": None, "node_ips": []}
            )
            _s["node_ips"].extend(k8s_clusters[k8s_cluster_name]["node_ips"])
            _s["node_port"] = service["node_port"]
            _s["namespace"] = service["namespace"]
            _s["template"] = service["template"]

            all_k8s_services[_service_name] = _s

    return all_k8s_services



import argparse
import os
import sys

from . import __version__
from .config import get_configuration_and_validate
from .outputs import load_output_config, generate_files
from .inputs import get_all_proper_k8s_services


def main():
    parser = argparse.ArgumentParser(prog="Hayde", description="Hayde")
    parser.add_argument("--config-file", required=True)
    parser.add_argument(
        "--version", action="version", version="%(prog)s {}".format(__version__)
    )
    args = parser.parse_args()

    config_file = args.config_file
    config = get_configuration_and_validate(config_file)

    loaded_output_config = load_output_config(config=config["output"])

    input_name = config["input"]["name"]
    if input_name == "k8s":
        all_proper_k8s_services = get_all_proper_k8s_services(
            k8s_configs=config["input"]
        )
        generate_files(
            output_config=loaded_output_config, k8s_services=all_proper_k8s_services
        )
    else:
        sys.exit(f"We only support k8s as input for now!")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hayde']

package_data = \
{'': ['*']}

install_requires = \
['Cerberus>=1.3.2,<2.0.0',
 'Jinja2>=2.11.2,<3.0.0',
 'PyYAML>=5.3.1,<6.0.0',
 'kubernetes>=12.0.1,<13.0.0']

entry_points = \
{'console_scripts': ['hayde = hayde.cli:main']}

setup_kwargs = {
    'name': 'hayde',
    'version': '0.1.1',
    'description': 'Hayde',
    'long_description': "# hayde [![PyPi version](https://img.shields.io/pypi/v/hayde.svg)](https://pypi.python.org/pypi/hayde/) [![PyPI pyversions](https://img.shields.io/pypi/pyversions/hayde.svg)](https://pypi.python.org/pypi/hayde/) [![](https://img.shields.io/gitlab/license/f9n/hayde.svg)](https://gitlab.com/f9n/hayde/blob/master/LICENSE)\n\nHayde\n\n# Installation\n\n```bash\n$ python3 -m pip install --user --upgrade hayde\n```\n\n# Usage\n\n```bash\n$ hayde -h\nusage: Hayde [-h] --config-file CONFIG_FILE [--version]\n\nHayde\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --config-file CONFIG_FILE\n  --version             show program's version number and exit\n```\n",
    'author': 'f9n',
    'author_email': 'f9n@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/f9n/hayde',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import io
import sys

import cerberus
import kubernetes as k8s
import yaml

CONFIG_FILE_SCHEMA = """
output:
    type: dict
    required: True
    schema:
        name:
            type: string
            required: True
        destination_dir:
            type: string
            required: True
        default_template:
            type: string
            required: True
        templates:
            type: dict
            required: True
            allow_unknown: True
        extras:
            type: dict
            allow_unknown: True
        filename_format:
            type: string
            required: True

input:
    type: dict
    required: True
    schema:
        name:
            type: string
            required: True
        service:
            type: dict
            schema:
                annotations:
                    type: dict
        contexts:
            type: list
            required: True
            schema:
                type: string
        node:
            type: dict
            schema:
                labels:
                    type: dict
                    required: True
"""


def read_configuration_file(filepath):
    try:
        with io.open(filepath, "r") as f:
            data = yaml.safe_load(f)
            return data
    except Exception as e:
        sys.exit(e)


def get_configuration(filepath):
    data = read_configuration_file(filepath)

    # Validate the config file
    v = cerberus.Validator()
    v.schema = yaml.safe_load(CONFIG_FILE_SCHEMA)
    v.validate(data)
    if v.errors:
        sys.exit(v.errors)

    return data


def get_configuration_and_validate(filepath):
    c = get_configuration(filepath)

    if c["input"]["name"] == "k8s":
        # Remove duplicate context names
        c["input"]["contexts"] = list(set(c["input"]["contexts"]))

        check_context_names_is_defined_on_kubeconfig(wanted_contexts=c["input"]["contexts"])

    return c


def check_context_names_is_defined_on_kubeconfig(wanted_contexts):
    local_k8s_contexts = get_k8s_contexts_on_local_machine()
    local_k8s_context_names = [ctx["name"] for ctx in local_k8s_contexts]
    for wanted_context in wanted_contexts:
        if not wanted_context in local_k8s_context_names:
            sys.exit("'{}' is not defined on kube-config file".format(wanted_context))


def get_k8s_contexts_on_local_machine():
    contexts, active_context = k8s.config.list_kube_config_contexts()
    if not contexts:
        sys.exit("Cannot find any context in kube-config file.")

    return contexts
